@@ -27,6 +27,11 @@ KEY__invite_or_poll_key = "invite_or_poll_key"
 KEY__invite_poll_key = "invite_poll_key"
 KEY__invite_code = "invite_code"
 KEY__invite_key_status = "invite_key_status"
+KEY__reset_key = "reset_key"
+KEY__reset_or_poll_key = "reset_or_poll_key"
+KEY__reset_poll_key = "reset_poll_key"
+KEY__reset_code = "reset_code"
+KEY__reset_key_status = "reset_key_status"
 KEY__parameters = "parameters"
 KEY__oauth_token = "oauth_token"
 KEY__template = "template"
@@ -59,6 +64,7 @@ KEY__mfa_key = "mfa_key"
 KEY__mfa_enabled = "mfa_enabled"
 KEY__username = "username"
 KEY__password = "password"
+KEY__default_database = "default_database"
 KEY__new_password = "new_password"
 KEY__new_password_confirm = "new_password_confirm"
 KEY__page = "page"
@@ -90,6 +96,7 @@ KEY__recipient_validation_view = "recipient_validation_view"
 KEY__email_template = "template"
 KEY__do_send_signup_email = "do_send_signup_email"
 KEY__allow_confirm_signup_attempt = "allow_confirm_signup_attempt"
+KEY__allow_reset_password = "allow_reset_password"
 KEY__already_signed_up_email_template = "existing_user_template"
 KEY__allow_signup = "allow_signup"
 KEY__email_account_name = "name"
@@ -97,6 +104,7 @@ KEY__email_sent = "sent"
 KEY__body = "body"
 KEY__default_email_signup_template = "default_email_signup_template"
 KEY__default_email_already_signed_up_template = "default_email_already_signed_up_template"
+KEY__default_reset_password_template = "default_reset_password_template"
 
 ATTR__deleted = "deleted"
 
@@ -143,8 +151,11 @@ ENVIRON__vault_key = "JAAQL_VAULT_PASSWORD"
 ENVIRON__email_credentials = "JAAQL_EMAIL_CREDENTIALS"
 ENVIRON__install_path = "INSTALL_PATH"
 
-EMAIL_PARAM__signup_key = "JAAQL_SIGNUP_KEY"
-EMAIL_PARAM__invite_code = "JAAQL_INVITE_CODE"
+EMAIL_PARAM__signup_key = "JAAQL_INVITE_KEY"
+EMAIL_PARAM__invite_code = "JAAQL_INVITE_SHORT_CODE"
+
+EMAIL_PARAM__reset_key = "JAAQL_RESET_KEY"
+EMAIL_PARAM__reset_code = "JAAQL_RESET_SHORT_CODE"
 
 CONFIG_KEY__server = "SERVER"
 CONFIG_KEY_SERVER__port = "port"
@@ -154,6 +165,10 @@ JWT_PURPOSE__pre_auth = "pre_auth"
 JWT_PURPOSE__connection = "connection"
 
 ERR__not_yet_installed = "JAAQL has not yet been installed!"
+ERR__user_public = "Cannot perform this action on a public user!"
+ERR__too_many_signup_attempts = "Too many signup attempts"
+ERR__too_many_reset_requests = "Too many reset requests"
+ERR__too_many_code_attempts = "Code disabled due to too many incorrect attempts. Please use the link in the email"
 
 PG_ENV__password = "POSTGRES_PASSWORD"
 
@@ -169,4 +184,4 @@ PORT__ems = 6061
 ENDPOINT__reload_accounts = "/reload-accounts"
 ENDPOINT__send_email = "/send-email"
 
-VERSION = "3.0.4"
+VERSION = "3.0.7"
