@@ -1,3 +1,5 @@
+import sys
+
 from jaaql.utilities.vault import Vault, DIR__vault
 from jaaql.db.db_interface import DBInterface
 import jaaql.utilities.crypt_utils as crypt_utils
@@ -61,7 +63,11 @@ FILE__was_installed = "was_installed"
 
 JWT__data = "data"
 
-ENVIRON__install_path = "INSTALL_PATH"
+WARNING__uninstall_allowed = "Due to installation parameters, the system can be uninstalled completely via the API. We do not recommend this being " \
+                             "used in production systems (the super user db password is required to perform uninstallation so it is not 'open') "
+
+DIR__apps = "apps"
+SEPARATOR__dir = "/"
 
 
 class JAAQLPivotData:
@@ -159,6 +165,12 @@ class BaseJAAQLModel:
 
         return ret
 
+    def set_jaaql_lookup_connection(self):
+        if self.vault.has_obj(VAULT_KEY__jaaql_lookup_connection):
+            jaaql_uri = self.vault.get_obj(VAULT_KEY__jaaql_lookup_connection)
+            address, port, db, username, password = DBInterface.fracture_uri(jaaql_uri)
+            self.jaaql_lookup_connection = create_interface(self.config, address, port, db, username, password, is_jaaql_user=True)
+
     def __init__(self, config, vault_key: str, migration_db_interface=None, migration_project_name: str = None,
                  migration_folder: str = None, is_container: bool = False, url: str = None):
         self.config = config
@@ -194,11 +206,16 @@ class BaseJAAQLModel:
             self.vault.insert_obj(VAULT_KEY__jwt_obj_crypt_key, jwt_obj_crypt_key.decode(crypt_utils.ENCODING__ascii))
 
         if self.vault.has_obj(VAULT_KEY__jaaql_lookup_connection):
+            if self.vault.has_obj(VAULT_KEY__allow_jaaql_uninstall):
+                self.uninstall_key = str(uuid.uuid4())
+                print(WARNING__uninstall_allowed, file=sys.stderr)
+                print("UNINSTALL KEY: " + self.uninstall_key)
+            else:
+                self.install_key = None
             self.has_installed = True
-            jaaql_uri = self.vault.get_obj(VAULT_KEY__jaaql_lookup_connection)
-            address, port, db, username, password = DBInterface.fracture_uri(jaaql_uri)
-            self.jaaql_lookup_connection = create_interface(self.config, address, port, db, username,
-                                                                        password, is_jaaql_user=True)
+
+        self.set_jaaql_lookup_connection()
+        if self.vault.has_obj(VAULT_KEY__jaaql_lookup_connection):
             run_migrations(self.jaaql_lookup_connection)
 
             if self.migration_db_interface is None:
@@ -207,6 +224,9 @@ class BaseJAAQLModel:
             run_migrations(self.jaaql_lookup_connection, migration_project_name, migration_folder=migration_folder,
                            update_db_interface=self.migration_db_interface)
 
+            if self.is_container:
+                self.jaaql_lookup_connection.close()  # Each individual class will have one
+
             self.email_manager = EmailManager()
         else:
             self.install_key = str(uuid.uuid4())
@@ -237,6 +257,14 @@ class BaseJAAQLModel:
 
         return paging_dict, parameters
 
+    def get_default_app_url(self):
+        return self.url + SEPARATOR__dir + DIR__apps
+
+    def replace_default_app_url(self, url_with_default: str):
+        if url_with_default is None:
+            return None
+        return url_with_default.replace("{{DEFAULT}}", self.get_default_app_url())
+
     def execute_paging_query(self, jaaql_connection: DBInterface, full_query: str, count_query: str, parameters: dict,
                              where_query: str, where_parameters: dict, decrypt_columns: list = None,
                              encryption_key: bytes = None):
